import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

public class Conway extends JPanel implements KeyListener, MouseListener, MouseMotionListener {
  int largeurCellule = 7;
  int iSelected = 0, jSelected = 0;
  boolean inside = false;
  Grille grille;
  long delay = 400;
  DynamiqueWorker worker = null;
  Color colVivant = new Color(255, 200, 0), colMorte = new Color(10, 100, 10);
  
  /**
   * Construit un panneau gérant l'affichage d'une grille et l'interaction avec le clavier
   * @param width largeur de la grille
   * @height hauteur de la grille
   */
  public Conway(int width, int height) {
    grille = new Grille(width, height);
    this.setFocusTraversalKeysEnabled(false);
    this.addKeyListener(this);
    this.addMouseListener(this);
    this.addMouseMotionListener(this);
    this.setFocusable(true);
    this.requestFocus();
    this.setPreferredSize(new Dimension(largeurCellule*width, largeurCellule*height));
    System.out.println("Appuyez sur :");
    System.out.println("ENTREE pour lancer/arrêter la dynamique");
    System.out.println("ESPACE pour vider la grille");
    System.out.println("r pour ré-initialiser la grille aléatoirement");
    System.out.println("+ pour accélérer");
    System.out.println("- pour ralentir");
  }
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    Graphics2D g2 = (Graphics2D) g;
    int i, j;
    for(i=0; i<grille.width; i++)
      for(j=0; j<grille.height; j++) {
      if(grille.etat[i][j]) g.setColor(colVivant);
      else g.setColor(colMorte);
      g.fillRect(largeurCellule*i, largeurCellule*j, largeurCellule, largeurCellule);
    }
    if(inside) {
        g.setColor(Color.RED);
        g.drawRect(largeurCellule*iSelected, largeurCellule*jSelected, largeurCellule, largeurCellule);
    }
  }
    
  // Implementation of KeyListener
  public void keyTyped(KeyEvent e) {}
  public void keyPressed(KeyEvent e) {
    if(e.getKeyChar()=='-')
    // Ralentit
      delay*=2;
    if(e.getKeyChar()=='+') {
    // Accelere
      delay/=2;
      if(delay == 0) delay =1;
    }
    // Lance ou arrete la dynamique
    if(e.getKeyCode()==KeyEvent.VK_ENTER)
      if(worker!=null) {
      worker.cancel(true);
      worker = null;
    } else {
      delay = 400;
      worker = new DynamiqueWorker();
      worker.execute();
    }
    // Vide la grille
    if(e.getKeyCode()==KeyEvent.VK_SPACE) {
      if(worker!=null) {
        worker.cancel(true);
        worker = null;
      }
      grille.vider();
      repaint();
    }
    // Re-initialise la grille aléatoirement
    if(e.getKeyChar()=='r') {
      if(worker!=null) {
        worker.cancel(true);
        worker = null;
      }
      grille.randomInit();
      repaint();
    }
  }
  public void keyReleased(KeyEvent e) {}
  
// Implementation of MouseListener
  public void mouseClicked(MouseEvent e) {
      int i, j;
      i = e.getX()/largeurCellule;
      j = e.getY()/largeurCellule;
      grille.etat[i][j] = !grille.etat[i][j];
      repaint();
  }
  public void mouseEntered(MouseEvent e) {
    inside = true;
  }
  public void mouseExited(MouseEvent e) {
    inside = false;
  }
  public void mousePressed(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}

// Implementation of MouseListener
  public void mouseMoved(MouseEvent e) {
    iSelected = e.getX()/largeurCellule;
    jSelected = e.getY()/largeurCellule;
    repaint();
  }
  public void mouseDragged(MouseEvent e) {}
    
  public static void main(String[] args) {
    JFrame frame = new JFrame("Jeu de la vie de Conway");
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    Conway c = new Conway(100, 100);
    
    frame.getContentPane().add(c);
    frame.setResizable(false);
    frame.pack();
    frame.setVisible(true);
  }
  
  class DynamiqueWorker extends SwingWorker<Void, Void> {
    public Void doInBackground() {
      long time = System.currentTimeMillis();
      
      while(!isCancelled()) {
        if(System.currentTimeMillis()-time>delay) {
          time = System.currentTimeMillis();
          grille.metAJour();
          repaint();
        }
      }
      return null;
    }
  }
}
